<?php

namespace Service\Integration;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Teknisa\ACL\Util\InstanceProvider;
use Util\Messages;
use Util\Utilities;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class ZmartIntegration {

    const CLIENT_ID = "CustomerPortal";
    const CLIENT_SECRET = "8c673f38ade42e5e3318c830d9522bd2";
    const ROUTE_LINK_BUDGET = '/linkDownloadBudgetSm';

    protected $entityManager;
    private $token;
    protected $ZMART_URL;
    protected $environment;
    protected $ZMART_LOGIN_URL;
   

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, $ZMART_URL, $ZMART_LOGIN_URL){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->ZMART_URL = $ZMART_URL;
        $this->ZMART_LOGIN_URL = $ZMART_LOGIN_URL;
    }

    public function requestZmartRoute($uri, $data = array(), $useLoginUrl = false){
        if(empty($this->token)){
            $this->token = $this->requestToken();
        }
        $data = json_encode($data);
        $url = $useLoginUrl ? ($this->ZMART_LOGIN_URL.$uri) : ($this->ZMART_URL.$uri);
        $header = array();
        $header[] = 'OAuth-Token: '.$this->token;
        $header[] = 'OAuth-SecretId: '.self::CLIENT_SECRET;
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Token-login: '.$this->environment->getToken();
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: '.strlen($data);
        $header[] = 'Expect:';
        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_TIMEOUT, 86400);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);
        curl_close($curl_session);
        $result = json_decode($result, true);
        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else if (!empty($result['messages'])) {
            throw new \Exception($result['messages'][0]['message']);
        }  else {
            $result = isset($result['dataset']) ? $result['dataset'] : null;
        }

        return $result;
    }

    private function requestToken(){
        $data = array(
            "CLIENT_ID" => self::CLIENT_ID,
            "CLIENT_SECRET" => self::CLIENT_SECRET
        );

        $data = json_encode($data);
        $url = $this->ZMART_URL.'/requestToken';
        $header = array();
        $header[] = 'Content-Type: application/json';
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Content-Length: '.strlen($data);

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POST, 1); 
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_SSL_VERIFYPEER , false);
        curl_setopt($curl_session, CURLOPT_USERAGENT, 'CustomerPortalOAuth');
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);

        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);

        curl_close($curl_session);
        $result = json_decode($result, true);
        $token = $result["dataset"]["token"][0];

        return $token;
    }

    private function getDataRequestFilter($filter = array(), $itemsPerPage = 500, $page = 1){
        $dataFilter = array(
            "requestType" => Request::TYPE_FILTER,
            "filter" => $filter,
            "itemsPerPage" => $itemsPerPage,
            "page" => $page
        );

        return $dataFilter;
    }

    private function getDataRequestRow($row) {
        $dataRow = array(
            "requestType" => Request::TYPE_ROW,
            "row" => $row
        );

        return $dataRow;
    }

    private function getDataRequestDataSet($dataSet) {
        $dataSet = array(
            "requestType" => Request::TYPE_DATA_SET,
            "dataset" => $dataSet
        );

        return $dataSet;
    }

    public function getZmartOperatorByNrorg($nrorg, $conditions, $page, $pageSize, $orderBy) {
        if(!Utilities::currentClienteIsTeknisa()){
            $result = $this->requestZmartRoute('/zmartOperator/findByNrorg',array(
                "NRORG" => $nrorg,
                "page" => $page,
                "pageSize" => $pageSize,
                "orderBy" => $orderBy,
                "conditions" => $conditions
            ));
            return $result["zmartOperator"];
        } else {
            return [];
        }
    }

    public function saveOperadorEmail($cdoperador, $dsemailoper, $nrorg, $cdoperultatu,$nrorgultatu){
        $result = $this->requestZmartRoute('/zmartOperator/saveOperadorEmail',array(
            "cdoperador" => $cdoperador,
            "dsemailoper" => $dsemailoper,
            "nrorg" => $nrorg,
            "cdoperultatu" => $cdoperultatu,
            "nrorgultatu" => $nrorgultatu 
        ));
        return $result;
    }

    public function requestNewZmartPassword($email){
        $result = $this->requestZmartRoute('/requestNewPassword',
        $this->getDataRequestFilter(array(
            array("name" => "EMAIL", "operator" => "=", "value"=>$email)
        )), true);
        return $result;
    }
  

}
